/*#include <vector>
#include <iostream>
#include <typeinfo>
#include <cxxabi.h>
#include <string>

#include "omp.h"
*/
#include "polymake/Vector.h"
#include "polymake/Matrix.h"


namespace DANGERZONE{
using namespace pm;
template <class E>
class Vector : public GenericVector<Vector<E>, E> {  
   typedef shared_array<E, AliasHandler<shared_alias_handler> > shared_array_type;
   const E* data;
   const int data_size;
   
public:
   /// element type
   typedef E value_type;
   /// reference to element type
   typedef E& reference;
   /// constant reference to element type
   typedef const E& const_reference;  

   Vector(const pm::Vector<E>& v):data(&v[0]), data_size(v.size()){}
   
    
   /// number of elements
   int size() const { return data_size; }

   /// true if Vector does not contain any elements
   bool empty() const { return data_size==0; }

  
   
   /// constant random access
   const E& operator[] (int i) const
   {
      if (POLYMAKE_DEBUG) {
         if (i<0 || i>=this->size())
            throw std::runtime_error("Vector::operator[] - index out of range");
      }
      return data[i];
   }
   
   
   typedef const E* const_iterator;

   const_iterator begin() const { return data; }
   const_iterator end() const { return data+data_size; }

   typedef std::reverse_iterator<const_iterator> const_reverse_iterator;

   const_reverse_iterator rbegin() const { return const_reverse_iterator(end()); }
   const_reverse_iterator rend() const { return const_reverse_iterator(begin()); }

   const_reference front() const { return begin()[0]; }
   const_reference back() const { return end()[-1]; }  
 
   
   typedef E* iterator;

   iterator begin() { return *data; }
   iterator end() { return *data+size(); }

   typedef std::reverse_iterator<iterator> reverse_iterator;

   reverse_iterator rbegin() { return reverse_iterator(end()); }
   reverse_iterator rend() { return reverse_iterator(begin()); }
 
   reference front() { return begin()[0]; }
   reference back() { return end()[-1]; }
   
};



template <typename E>
class Matrix_base {
protected:
   struct dim_t {
      int dimr, dimc;
      dim_t() : dimr(0), dimc(0) {}
      dim_t(int r, int c) : dimr(c? r:0), dimc(r? c:0) {
         if (POLYMAKE_DEBUG) {
            if (dimr<0 || dimc<0)
               throw std::runtime_error("Matrix_base::dim_t out of range");
         }
      }
   };
   
   struct data_t{
	
	dim_t dim;
	E* data; 
	data_t(pm::Matrix<E>& m):dim(m.rows(), m.cols()),data(&m(0,0)){}
	
	dim_t& get_prefix(){return dim;}
	const dim_t& get_prefix() const {return dim;}
	size_t size() const {return dim.dimr*dim.dimc;}
   };
  
  data_t data;
  Matrix_base(pm::Matrix<E>& m):data(m){}
  
  typedef E value_type;
  typedef E& reference;
  typedef const E& const_reference;
  
  
   template <bool,typename> friend class pm::matrix_line_factory;
   friend class pm::ConcatRows<Matrix_base>;
//  template <bool,typename> friend class matrix_line_factory;
//  friend class ConcatRows<Matrix_base>;
//  template <typename, int> friend class alias;
 
};  
template <typename E>
class Matrix : public GenericMatrix<Matrix<E>, E>,
               protected Matrix_base<E> {
   typedef Matrix_base<E> base;
   typedef typename base::dim_t dim_t;


public:
   typedef E value_type;
   typedef E& reference;
   typedef const E& const_reference;
   
   Matrix(pm::Matrix<E>& m):base(m){}
   
   int rows() const { return this->data.dim.dimr; }
   int cols() const { return this->data.dim.dimc; }
   
   reference operator() (int i, int j)
   {
      if (POLYMAKE_DEBUG) {
         if (i<0 || i>=this->rows() || j<0 || j>=this->cols())
            throw std::runtime_error("Matrix::operator() - index out of range");
      }
      return (this->data.data)[i*cols()+j];
   }
   const_reference operator() (int i, int j) const
   {
      if (POLYMAKE_DEBUG) {
         if (i<0 || i>=this->rows() || j<0 || j>=this->cols())
            throw std::runtime_error("Matrix::operator() - index out of range");
      }
      return (this->data.data)[i*cols()+j];
   }
   
   friend class pm::ConcatRows<Matrix>;
   template <typename,typename> friend class pm::GenericMatrix;
   friend class pm::Rows<Matrix>;
   friend class pm::Cols<Matrix>;
   template <typename, typename> friend class pm::RowChain;
   template <typename, typename> friend class pm::ColChain;
};



}//DANGERZONE
namespace pm{

template <typename E>
struct check_container_feature<DANGERZONE::Matrix<E>, FlatStorage> : True {};

template <typename E>
class ConcatRows< DANGERZONE::Matrix_base<E> >
   : public plain_array< ConcatRows< DANGERZONE::Matrix_base<E> >, E >,
     public GenericVector< ConcatRows< DANGERZONE::Matrix_base<E> >, E> {
protected:
   ~ConcatRows();
   DANGERZONE::Matrix_base<E>& hidden() { return *reinterpret_cast<DANGERZONE::Matrix_base<E>*>(this); }
   const DANGERZONE::Matrix_base<E>& hidden() const { return *reinterpret_cast<const DANGERZONE::Matrix_base<E>*>(this); }
public:
   int size() const { return hidden().data.size(); }
   E& front() { return hidden().data.data[0]; }
   const E& front() const { return hidden().data.data[0]; }

   ConcatRows& operator= (const ConcatRows& m)
   {
      return ConcatRows::generic_type::operator=(m);
   }
   using ConcatRows::generic_type::operator=;
};

template <typename E>
class ConcatRows< DANGERZONE::Matrix<E> >
   : public ConcatRows< DANGERZONE::Matrix_base<E> > {
public:
   using ConcatRows::generic_type::operator=;
};


template <typename E>
class Rows< DANGERZONE::Matrix<E> >
   : public modified_container_pair_impl< Rows< DANGERZONE::Matrix<E> >,
                                          list( Container1< constant_value_container< DANGERZONE::Matrix_base<E>& > >,
                                                Container2< series >,
                                                Operation< matrix_line_factory<true> >,
                                                MasqueradedTop ) > {
protected:
   ~Rows();
public:
   constant_value_container< DANGERZONE::Matrix_base<E>& > get_container1()
   {
      return this->hidden();
   }
   const constant_value_container< const DANGERZONE::Matrix_base<E>& > get_container1() const
   {
      return this->hidden();
   }
   series get_container2() const
   {
      const DANGERZONE::Matrix<E>& me=this->hidden();
      return series(0, me.rows(), me.cols());
   }
   void resize(int n)
   {
      DANGERZONE::Matrix<E>& me=this->hidden();
      me.resize(n, me.cols());
   }
};

template <typename E>
class Cols< DANGERZONE::Matrix<E> >
   : public modified_container_pair_impl< Cols< DANGERZONE::Matrix<E> >,
                                          list( Container1< constant_value_container< DANGERZONE::Matrix_base<E>& > >,
                                                Container2< sequence >,
                                                Operation< matrix_line_factory<false> >,
                                                MasqueradedTop ) > {
protected:
   ~Cols();
public:
   constant_value_container< DANGERZONE::Matrix_base<E>& > get_container1()
   {
      return this->hidden();
   }
   const constant_value_container< const DANGERZONE::Matrix_base<E>& > get_container1() const
   {
      return this->hidden();
   }
   sequence get_container2() const
   {
      return sequence(0, this->hidden().cols());
   }
   void resize(int n)
   {
      DANGERZONE::Matrix<E>& me=this->hidden();
      me.resize(me.rows(), n);
   }
};

}//pm

